#! /usr/bin/env bash
set -euo pipefail
IFS=$'\n'

function tf_usage {
  cat >&2 <<EOF
$(basename "$0") [-u] [-v] <directory>
  -u  Check dependencies for upgrade during init.
  -v  Verbose output.

Find all terraform modules under the current directory and run linting and
validation. Full validation and linting is only run on modules under the given
directory, others are only checked for expected formatting.

Any directory containing "*.tf" files is considered a terraform module.
EOF
  exit 64
}

function if_verbose {
  if [ -n "${VERBOSE}" ]; then
    printf '%s\n' "$(tr '\n' ' ' <<<"$*")"
  fi
  eval "$@"
}

function tf_directories {
  local under="${1:-.}"
  find "${under}" \( -name '.*' \! -name '.' -o -path './node_modules' \) -prune -o \
    -name '*.tf' -type f -exec dirname \{\} \; | \
    sort -u
}

function tf_check_fmt {
  local directory="$1"

  (
    if_verbose cd "${directory}"
    if_verbose terraform fmt --check=true --diff --recursive
  )
}

function tf_check {
  local directory="$1"

  (
    if_verbose cd "${directory}"
    if_verbose terraform init -backend=false -input=false "${TERRAFORM_INIT_ARGS[@]}"

    if_verbose terraform validate
    if_verbose tflint ./*.tf
  )
}

export TF_IN_AUTOMATION=true
declare -a TERRAFORM_INIT_ARGS
if [ -d /usr/local/bin/terraform-plugins ]; then
  TERRAFORM_INIT_ARGS+=('-plugin-dir=/usr/local/bin/terraform-plugins')
fi
VERBOSE=

while getopts ":uv" OPT; do
  case "${OPT}" in
    u)
      # Check dependencies for upgrade during init.
      TERRAFORM_INIT_ARGS+=('-upgrade=true')
      ;;
    v)
      # Verbose output.
      VERBOSE=true
      ;;
    \?)
      # Invalid argument.
      tf_usage
      ;;
  esac
done

# PATH="$(cd "$(dirname "$0")" && pwd):${PATH}"
shift "$(( OPTIND - 1 ))"
if [ "$#" -gt 1 ]; then
  tf_usage
fi

VALIDATE_UNDER="${1:-}"

if [ -n "${CI_JOB_TOKEN:-}" ]; then
  echo -e "machine gitlab.bit9.local\nlogin gitlab-ci-token\npassword ${CI_JOB_TOKEN}" > ~/.netrc
  git config --global url.https://gitlab.bit9.local/.insteadOf ssh://git@gitlab.bit9.local/
  echo "Configured: gitlab-ci-token"
fi

for DIR in $(tf_directories); do
  tf_check_fmt "${DIR}"
done

for DIR in $(tf_directories "${VALIDATE_UNDER}"); do
  echo "==> tflinting"
  tf_check "${DIR}"
done
