#! /usr/bin/env bash
set -euo pipefail
IFS=$'\n'

export PATH=/usr/local/bin/terraform12/:$PATH

# Mac OS Support for tf12
export PATH=/usr/local/opt/terraform/bin/:$PATH

function tf_usage {
  cat >&2 <<EOF
$(basename "$0") [-u] [-v] <directory>
  -u  Check dependencies for upgrade during init.
  -v  Verbose output.

Find all terraform modules under the current directory and run linting and
validation. Full validation and linting is only run on modules under the given
directory, others are only checked for expected formatting.

Any directory containing "*.tf" files is considered a terraform module.
EOF
  exit 64
}

function if_verbose {
  if [ -n "${VERBOSE}" ]; then
    printf '%s\n' "$(tr '\n' ' ' <<<"$*")"
  fi
  eval "$@"
}

function tf_directories {
  find . \( -name '.*' \! -name '.' -o -path './node_modules' \) -prune -o \
    -name '*.tf' -type f -exec dirname \{\} \; | \
    sort -u | grep -v '/aws-auth/' # tf 11 fmt doesn't work for tf 12
}

function tf_check_fmt {
  set -e
  local directory="$1"

  (
    if_verbose terraform fmt -diff=true -check=true "${directory}"
    if_verbose terraform fmt -diff=true -check=true "${directory}/config"

    for config in "$directory"/config/*.tfvars; do
      if_verbose tflint --var-file="${config}" "${directory}"
    done
  )
}

export TF_IN_AUTOMATION=true
VERBOSE=

while getopts ":v" OPT; do
  case "${OPT}" in
    v)
      # Verbose output.
      VERBOSE=true
      ;;
    \?)
      # Invalid argument.
      tf_usage
      ;;
  esac
done

PATH="$(cd "$(dirname "$0")" && pwd):${PATH}"
shift "$(( OPTIND - 1 ))"
if [ "$#" -gt 1 ]; then
  tf_usage
fi

# Export these functions/variables for parallel to use
export VERBOSE
export -f tf_check_fmt tf_directories if_verbose

parallel --halt soon,fail=1 --group tf_check_fmt ::: "$(tf_directories)"
